
import java.io.*;
import java.awt.Color;
import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import java.text.*;
import java.util.ArrayList;
import java.util.List;

/*the class represents the last results window containing a drawing of the best ans*/
public class SequenceResult {

    private Data data;

    private int index;
    private int counter;
    private String shap;
    private String dg;
    private String neut;
    private String sequence;
    public SequenceResult(Data data,  int counter){
        this.data = data;
        this.counter = counter;
    }

    public void init(Ans ans, int index) throws IOException, InterruptedException 
    {
        shap = ans.shapiro;
        dg = ans.dg;
        sequence = ans.seq;
        neut = ans.neut;
        this.index = index;

        try
        {
		Utils.createWildTypePlot(this.data.getRecDB());  
		Utils.createRnaFold(ans);
        }
        catch (Exception e) {
		System.err.println("SequenceResult Error");
		e.printStackTrace();
        }


	File f_old = new File("tmp" + File.separator + "rna"+ this.index +counter+".ps");
	f_old.delete();
	File f = new File("rna.ps");
 	if (!f.renameTo(f_old))
		throw new IOException("Failed to move file: " + f);

	String os = System.getProperty("os.name");
	String programName;
	if (!os.startsWith("Windows"))	
		programName = "gs";
	else
		programName = "gswin32c.exe";
	List<String> commandAndOptions_ProcessBuilder = new ArrayList<String>();
    	commandAndOptions_ProcessBuilder.add(programName);
	commandAndOptions_ProcessBuilder.add("-q");
	commandAndOptions_ProcessBuilder.add("-sDEVICE=jpeg");
	commandAndOptions_ProcessBuilder.add("-sPAPERSIZE=a4");
	commandAndOptions_ProcessBuilder.add("-dBATCH");
	commandAndOptions_ProcessBuilder.add("-dNOPAUSE");
	commandAndOptions_ProcessBuilder.add("-sOutputFile=tmp" + File.separator + "rna"+ this.index + counter +".jpeg");
	commandAndOptions_ProcessBuilder.add("tmp" + File.separator + "rna"+ this.index +counter+".ps");
	ProcessBuilder pb = new ProcessBuilder(commandAndOptions_ProcessBuilder);
	pb.redirectErrorStream(true);
	Process process = pb.start();		
	process.waitFor();
    }

    public void showFrame(int index) throws IOException
    {
        DecimalFormat df = new DecimalFormat("0.00");

        /*creating a frame*/
        JFrame seq  = new JFrame ();//box for Clustering Resolution
        seq.setSize(1000,700);

        java.awt.GridBagConstraints gridBagConstraints;
        seq.getContentPane().setLayout(new java.awt.GridBagLayout());

	JTextField lab3 = new JTextField("The sequence: " + sequence);
	lab3.setEditable(false);
	lab3.setBorder(null);
	lab3.setDisabledTextColor(Color.black);

        JLabel lab5 = new JLabel("Free energy: "+ df.format(Double.parseDouble(dg))+" kcal/mol (Target: " + 
				((data.getTarget() == -1000.0) ? "no target" : df.format(data.getTarget())+" kcal/mol") + ")");
        JLabel lab6 = new JLabel("Neutrality "+ df.format(Double.parseDouble(neut))+ " (Target: " + 
				((data.getNeutrality() == -1000.0) ? "no target" : df.format(data.getNeutrality())) + ")");
        JLabel lab7 = new JLabel("Shapiro representation: "+ shap);

        JLabel lab10 = new JLabel("Target:  ");
        JLabel lab11 = new JLabel("Result:  ");


	BufferedImage icon = ImageIO.read(new File("tmp" + File.separator + "rna"+ this.index +counter+".jpeg"));
	icon = icon.getSubimage(69,195,452,452);
	JLabel labResult = new JLabel(new ImageIcon(icon));
        JPanel arguments2 = new JPanel();
        arguments2.add(labResult);

        BufferedImage wildIcon = ImageIO.read(new File("tmp" + File.separator + "rnaWild.jpeg"));
	wildIcon = wildIcon.getSubimage(69,195,452,452);        
	JLabel labWild = new JLabel(new ImageIcon(wildIcon));
        JPanel arguments1 = new JPanel();
        arguments1.add(labWild);

        JPanel argumentsTitles1 = new JPanel();
        argumentsTitles1.add(lab10);

        JPanel argumentsTitles2 = new JPanel();
        argumentsTitles2.add(lab11);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        seq.getContentPane().add(argumentsTitles1, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        seq.getContentPane().add(arguments1, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 20;
        gridBagConstraints.gridy = 10;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        seq.getContentPane().add(argumentsTitles2, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 20;
        gridBagConstraints.gridy = 10;

        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        seq.getContentPane().add(arguments2, gridBagConstraints);

        JPanel arguments = new JPanel();
        JLabel lab0 = new JLabel("   ");
        arguments.add(lab0);
        arguments.add(lab0);
        arguments.add(lab3);
        arguments.add(lab7);
        arguments.add(lab5);
        arguments.add(lab6);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 20;
        gridBagConstraints.gridwidth = 8;
        gridBagConstraints.ipadx = 60;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(4, 4, 4, 4);
        seq.getContentPane().add(arguments, gridBagConstraints);

        arguments.setLayout( new BoxLayout( arguments, BoxLayout.PAGE_AXIS ) );
        arguments.setBorder( BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder( "Result sequence data" ),
                BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) ) );
        //arguments.setPreferredSize( new Dimension( 270, 200 ) );

        /* arranging the frame*/
        seq.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);

        seq.setTitle("The secondary structure of the sequence");
        seq.setVisible(true);
        if(!seq.isDisplayable())
        {
            seq.dispose();
        }
    }
}
