
import java.io.*;
import java.text.MessageFormat;
import java.util.*;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;

class Utils
{
    public static void createWildTypePlot(String db) throws IOException, InterruptedException
    {
       String wildType = "";
        for(int i= 0 ; i<db.length() ; i++){
            wildType= wildType+"N";
        }

	String fileInput = "tmp/wildSeq.txt";
        FileWriter wildfstream = new FileWriter(fileInput);
        BufferedWriter wildout = new BufferedWriter(wildfstream);
        wildout.write(wildType);
        wildout.write("\n" + db);
        wildout.close();

	String os = System.getProperty("os.name");
	String programName;
	if (!os.startsWith("Windows"))	
		programName = "bin" + File.separator + "RNAplot";
	else
		programName = "bin" + File.separator + "RNAplot.exe";
	ProcessBuilder pb = new ProcessBuilder(programName);
	pb.redirectErrorStream(true);
	Process process = pb.start();		        
	final InputStream is = process.getInputStream();
	// the background thread watches the output from the process
	new Thread(new Runnable() {
	public void run() {
		try {
			BufferedReader reader =
			new BufferedReader(new InputStreamReader(is));
			String line;
			while ((line = reader.readLine()) != null) {
				System.out.println(line);
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				is.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	}).start();

	final OutputStream pos = process.getOutputStream();
	final PrintWriter pw = new PrintWriter(pos);
	final InputStream fis = new FileInputStream(fileInput);
	final BufferedReader fileBr = new BufferedReader(new InputStreamReader(fis));

	new Thread(new Runnable() {
	public void run() {
		try {
			String lineRead;
			while ((lineRead = fileBr.readLine()) != null) {
				pw.print(lineRead + "\n");
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (pw != null) {
				pw.close();
			}
			if (fileBr != null) {
				try {
					fileBr.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	}).start();

	process.waitFor();

	File f_old = new File("tmp" + File.separator + "rnaWild.ps");
	f_old.delete();
	File f = new File("rna.ps");
 	if (!f.renameTo(f_old))
		throw new IOException("Failed to move file: " + f);

	if (!os.startsWith("Windows"))	
		programName = "gs";
	else
		programName = "gswin32c.exe";
	String[] commandAndOptions_ProcessBuilder = {programName, "-q", "-sDEVICE=jpeg", "-sPAPERSIZE=a4", "-dBATCH", "-dNOPAUSE", "-sOutputFile=tmp" + File.separator + "rnaWild.jpeg",
							"tmp" + File.separator + "rnaWild.ps"};
	pb = new ProcessBuilder(commandAndOptions_ProcessBuilder);
	pb.redirectErrorStream(true);
	process = pb.start();		
	process.waitFor();
    }


   public static String createRnaFold(Ans newAns) throws IOException, InterruptedException
   {
	String fileInput = "tmp" + File.separator + "seq.txt";
	File f_old = new File(fileInput);
	f_old.delete();
	FileWriter f_seq = new FileWriter(fileInput);
	BufferedWriter f_out = new BufferedWriter(f_seq);
	f_out.write(newAns.seq);
	f_out.close();

	String os = System.getProperty("os.name");
	String programName;
	if (!os.startsWith("Windows"))	
		programName = "bin" + File.separator + "RNAfold";
	else
		programName = "bin" + File.separator + "RNAfold.exe";
	java.util.List<String> commandAndOptions_ProcessBuilder = new ArrayList<String>();
	commandAndOptions_ProcessBuilder.add(programName);
	commandAndOptions_ProcessBuilder.add("-noLP");
	ProcessBuilder pb = new ProcessBuilder(commandAndOptions_ProcessBuilder);
	pb.redirectErrorStream(true);
	Process process = pb.start();		        
	final InputStream is = process.getInputStream();
	final Ans ans = new Ans();
	// the background thread watches the output from the process
	Thread t1 = new Thread(new Runnable() {
	public void run() {
		try {
			BufferedReader reader =
			new BufferedReader(new InputStreamReader(is));
			reader.readLine();
			StringTokenizer st = new StringTokenizer(reader.readLine());
			ans.dot = st.nextToken();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				is.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	});	
	t1.start();

	final OutputStream pos = process.getOutputStream();
	final PrintWriter pw = new PrintWriter(pos);
	final InputStream fis = new FileInputStream(fileInput);
	final BufferedReader fileBr = new BufferedReader(new InputStreamReader(fis));

	new Thread(new Runnable() {
	public void run() {
		try {
			String lineRead;
			while ((lineRead = fileBr.readLine()) != null) {
				pw.print(lineRead + "\n");
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (pw != null) {
				pw.close();
			}
			if (fileBr != null) {
				try {
					fileBr.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	}).start();

	process.waitFor();
	t1.join();
	return ans.dot;
   }

   public static int calculateRnaDistance(Ans newAns, String db) throws IOException, InterruptedException
   {
	String fileInput = "tmp" + File.separator + "dots.txt";
	File f_old = new File(fileInput);
	f_old.delete();
	FileWriter f_dot = new FileWriter(fileInput);
	BufferedWriter f_dot_out = new BufferedWriter(f_dot);
	f_dot_out.write(db + " \n");
	f_dot_out.write(newAns.dot);
	f_dot_out.close();

	String os = System.getProperty("os.name");
	String programName;
	if (!os.startsWith("Windows"))	
		programName = "bin" + File.separator + "RNAdistance";
	else
		programName = "bin" + File.separator + "RNAdistance.exe";
	ProcessBuilder pb = new ProcessBuilder(programName);
	pb.redirectErrorStream(true);
	Process process = pb.start();		        
	final InputStream is = process.getInputStream();
	final Ans ans = new Ans();
	// the background thread watches the output from the process
	Thread t1 = new Thread(new Runnable() {
	public void run() {
		try {
			BufferedReader reader =
			new BufferedReader(new InputStreamReader(is));
			   StringTokenizer st = new StringTokenizer(reader.readLine());
			   st.nextToken();
			   ans.dist = Integer.parseInt(st.nextToken());
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				is.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	});
	t1.start();

	final OutputStream pos = process.getOutputStream();
	final PrintWriter pw = new PrintWriter(pos);
	final InputStream fis = new FileInputStream(fileInput);
	final BufferedReader fileBr = new BufferedReader(new InputStreamReader(fis));

	new Thread(new Runnable() {
	public void run() {
		try {
			String lineRead;
			while ((lineRead = fileBr.readLine()) != null) {
				pw.print(lineRead + "\n");
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			if (pw != null) {
				pw.close();
			}
			if (fileBr != null) {
				try {
					fileBr.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}
	}).start();

	process.waitFor();
	t1.join();
	return ans.dist;
   }

}
